/******************************/
/*                            */
/*       assignment2.c        */
/*  Written by Tzu-Kai Chou   */
/*                            */
/******************************/

#include <stdio.h>
#include <mpi.h>
#include <math.h>

#define MAX_PROCESSOR 20

int main(int argc, char *argv[])
{
   double gather[MAX_PROCESSOR];
   double PI25DT=3.14159265358979323842643;
 
   int num_intervals, myid, num_procs, i;
   double mypi, h, x;
   double pi=0.0, sum=0.0;

   FILE *fp;
   char str[81];

   MPI_Init(&argc, &argv);
   MPI_Comm_size(MPI_COMM_WORLD, &num_procs);
   MPI_Comm_rank(MPI_COMM_WORLD, &myid);

   /* Program get input data by reading the file input.dat      */
   /* Only process 0 go to read num_intervals then broadcast.   */
   /* Thus we can make sure every process got the same data.    */
   if(myid==0)
   {
     fp=fopen("assignment2/input.dat", "r");
     if(fp==NULL)                /* The file can not be opened. */
        num_intervals=1024;      /* Use default num_intervals */
     else
     {
        fgets(str, 80, fp);
        num_intervals=atoi(str); /* string-->integer */
        if(num_intervals<=0)     /* If num_intervals is invalid. */
           num_intervals=1024;   /* Use default num_intervals */
     } 
     fclose(fp);
   } 
   MPI_Bcast(&num_intervals,1,MPI_INT,0,MPI_COMM_WORLD);

   /* Each process compute the value of the integral in its own interval */
   h=1.0/(double)num_intervals; 
   for(i=myid+1; i<=num_intervals; i+=num_procs)
   {
      x=h*((double)i-0.5);
      sum+=4.0/(1.0+x*x);
   }
   mypi=h*sum;

   /* All the results going to process 0 */ 
   MPI_Gather(&mypi,1,MPI_DOUBLE,gather,num_procs,MPI_DOUBLE,0,MPI_COMM_WORLD);

   if(myid==0)  /* Sum up the results to get the PI */
   {
     for(i=0; i<num_procs; i++) 
        pi+=gather[i];
     printf("node%d PI is approximately %.16f\n",myid,pi);
     printf("      The error is %.16f\n",fabs(pi-PI25DT));
     printf("      The number of intervals is %d\n",num_intervals);
   }
 
   MPI_Bcast(&pi,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
 
   if(myid!=0)
     printf("node%d receive broadcast PI=%.16f\n",myid,pi);
 
   MPI_Finalize();
}
